<?php
if(!defined('ABSPATH')){
	die('Hacking Attempt!');
}

/**
 * Retrieves the country associated with a given store URL.
 *
 * This function queries the database to fetch the country associated with the provided store URL
 * from the 'quickaffilink_config' table.
 *
 * @param string $store_url The URL of the store.
 * @return string|false The name of the country if found, or false if not found.
 */
function quickaffilink_store_country_by_url($store_url) {
    global $wpdb;

    // Construct the table name with the WordPress database prefix
    $table_name_1 = $wpdb->prefix . 'quickaffilink_config';

   
	// Prepare and execute the query to retrieve the store country
    $store_country = $wpdb->get_var($wpdb->prepare( "SELECT country_name FROM $table_name_1 WHERE store_url = %s", $store_url ));

    // Return the store country
    return $store_country;
}


/**
 * Prepopulates the configuration table with default values.
 *
 * This function prepopulates the 'quickaffilink_config' table in the database with default values
 * for Amazon stores, as defined in the `quickaffilink__default_options()` function.
 */
function quickaffilink_prepopulate_config_table() {
    global $wpdb;

    // Construct the table name with the WordPress database prefix
    $table_name_2 = $wpdb->prefix . 'quickaffilink_config';

    // Define default Amazon stores
    $amazon_stores = quickaffilink__default_options();

    // Prepopulate the table with default configurations
    foreach ($amazon_stores as $country_name => $store_url) {
        // Extract top-level domain (TLD) from the store URL
        $dot_position = strpos($store_url, '.'); // Find the position of the first dot
        if ($dot_position !== false) {
            $tld = substr($store_url, $dot_position); // Get everything from the first dot to the end of the store URL
        } else {
            // Handle the case where there is no dot in the store URL
            // This could be an error condition, log or handle it appropriately
        }
        
        // Determine if the store is the default (Amazon.com)
        $is_default = ($tld === '.com') ? 1 : 0; // Set is_default to true for Amazon.com, false for others
        
        // Insert the configuration into the database
        $wpdb->insert(
            $table_name_2,
            array(
                'tld' => $tld,
                'country_name' => $country_name,
                'store_url' => $store_url,
                'is_default' => $is_default,
                'created_by' => get_current_user_id(),
                'creation_date' => current_time('mysql')
            )
        );
    }
}

/**
 * Generate hidden input fields from a nested array.
 *
 * This function generates hidden input fields from a nested array. It recursively traverses
 * the array and generates input fields for each key-value pair.
 *
 * @param array $array The array from which to generate hidden input fields.
 * @param string $parentKey Optional. The parent key for nested arrays.
 * @return string The generated hidden input fields.
 */
function quickaffilink_generateHiddenFields($array, $parentKey = '') {
    $hiddenFields = '';
    foreach ($array as $key => $value) {
        // Generate the field name
        $fieldName = $parentKey ? $parentKey . '_' . ($key + 1) . '' : $key;
        
        if (is_array($value)) {
            // If the value is an array, recursively call the function
            $hiddenFields .= quickaffilink_generateHiddenFields($value, $fieldName);
        } else {
            // Generate the hidden input field
          
			if( $fieldName == "product_overview"){
				$hiddenFields .= '<input type="hidden" id="' . $fieldName . '" value="' . esc_attr(htmlspecialchars(addslashes($value))) . '">';					
			}
			else{
				$hiddenFields .= '<input type="hidden" id="' . $fieldName . '" value="' . esc_attr($value) . '">';		
			}
		
        }
    }
    return $hiddenFields;
}

/**
 * Formats the time difference between a given timestamp and the current time.
 *
 * @param string $timestamp The timestamp to calculate the time difference from.
 * @return string The formatted time difference string.
 */

function quickaffilink_formatTimeDifference($timestamp) {
    // Get the current time from the database in UTC
    $now_query = current_time('mysql');
    $now = strtotime($now_query); // Convert it to a Unix timestamp
    
    // Convert the given timestamp to a Unix timestamp
    $past = strtotime($timestamp);
    
    // Check if the timestamp is in the future
    if ($past > $now) {
        return esc_html__('In the future', 'quickaffilink'); // Handle future timestamps appropriately
    }
    
    // Calculate the time difference in seconds
    $timeDifference = $now - $past;

    // Determine the appropriate time format based on the time difference
    if ($timeDifference < 60) {
        return esc_html__('Just now', 'quickaffilink');
    } elseif ($timeDifference < 3600) {
        $minutes = floor($timeDifference / 60);
        return $minutes . ' ' . ($minutes === 1 ? esc_html__('minute ago', 'quickaffilink') : esc_html__('minutes ago', 'quickaffilink'));
    } elseif ($timeDifference < 86400) {
        $hours = floor($timeDifference / 3600);
        return $hours . ' ' . ($hours === 1 ? esc_html__('hour ago', 'quickaffilink') : esc_html__('hours ago', 'quickaffilink'));
    } elseif ($timeDifference < 31536000) { // Less than a year
        $days = floor($timeDifference / 86400);
        return $days . ' ' . ($days === 1 ? esc_html__('day ago', 'quickaffilink') : esc_html__('days ago', 'quickaffilink'));
    } else { // More than a year
        $years = floor($timeDifference / 31536000);
        return $years . ' ' . ($years === 1 ? esc_html__('year ago', 'quickaffilink') : esc_html__('years ago', 'quickaffilink'));
    }
}



/**
 * Function to fetch product data by ID
 *
 * This function queries the database to retrieve product data based on the provided product ID.
 *
 * @param int $product_id The unique identifier of the product.
 * @return array|null An associative array containing the product data if found, or null if no data is found.
 */

function quickaffilink_get_product_data($product_id) {
    global $wpdb;
    $table_name_3 = $wpdb->prefix . 'quickaffilink_products';
	 return $wpdb->get_row($wpdb->prepare("SELECT * FROM $table_name_3 WHERE id = %d", $product_id));

}

/**
 * Appends affiliate tag to a product URL based on the store URL.
 *
 * This function retrieves the affiliate tag associated with the provided store URL from the database.
 * If an affiliate tag is found, it parses the product URL to remove any existing 'tag' attribute,
 * rebuilds the query string without the 'tag' attribute, and reconstructs the product URL.
 * Finally, it appends the retrieved affiliate tag to the product URL and returns the modified URL.
 *
 * @param string $store_url The URL of the store associated with the affiliate tag.
 * @param string $product_url The URL of the product to which the affiliate tag will be appended.
 * @return string The modified product URL with the affiliate tag appended.
 */
function quickaffilink_append_affiliate_tag_to_url($store_url, $product_url) {
    global $wpdb;
    $table_name_4 = $wpdb->prefix . 'quickaffilink_config';
    $affiliate_tag = '';

    // Prepare and execute the query to retrieve the affiliate tag for the provided store URL
	$result = $wpdb->get_var(
    $wpdb->prepare("SELECT affiliate_tag FROM $table_name_4 WHERE store_url = %s", $store_url));

    // Check if an affiliate tag was found for the provided store URL
    if ($result) {
        $affiliate_tag = $result;

        // Parse the product URL to remove any existing 'tag' attribute
		$parsed_url = wp_parse_url($product_url);
		parse_str($parsed_url['query'], $query_params);
		unset($query_params['tag']);
        // Rebuild the query string without the 'tag' attribute
        $new_query = http_build_query($query_params);

        // Reconstruct the product URL with the new query string
        $product_url = $parsed_url['scheme'] . '://' . $parsed_url['host'] . $parsed_url['path'];
        if ($new_query) {
            $product_url .= '?' . $new_query;
        }

        // Append the affiliate tag to the product URL
        $product_url .= '&tag=' . urlencode($affiliate_tag);
    } else {
        // Log an error if no affiliate tag is found for the provided store URL
        error_log("No affiliate tag found for store URL: $store_url");
    }

    return $product_url;
}

/**
 * Trims a given title to a specified number of words.
 *
 * @param string $title The title string to be trimmed.
 * @param int $words Optional. The maximum number of words to keep in the trimmed title. Default is 12.
 * @return string The trimmed title string.
 */
function quickaffilink_trim_title($title, $words = 12) {
    $title_words = explode(' ', $title);
    if (count($title_words) > $words) {
        $title = implode(' ', array_slice($title_words, 0, $words)) . '...';
    }
    return $title;
}


/**
 * Extracts a specified number of list items (<li>) from the provided HTML content.
 *
 * @param string $html The HTML content from which list items will be extracted.
 * @param int $count Optional. The number of list items to extract. Default is 2.
 * @return string A string containing the extracted list items wrapped within <ul> tags.
 */
function quickaffilink_getListItems($html, $count = 2) {
    // Check if the input HTML is empty or not a string
    if (empty($html) || !is_string($html)) {
        return '';
    }

    $dom = new DOMDocument('1.0', 'UTF-8');

    // Suppress warnings for loadHTML errors

	$dom = new DOMDocument();
	libxml_use_internal_errors(true); // Suppress warnings from invalid HTML
	$dom->loadHTML($html, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
    libxml_clear_errors();

    // Get the first ul element, if it exists
    $ul_element = $dom->getElementsByTagName('ul')->item(0);

    // Check if ul element exists
    if (!$ul_element) {
        return $html; // Return the passed contents as is if <ul> element is not found
    }

    // Get li elements up to the specified count
    $li_elements = $ul_element->getElementsByTagName('li');
    
    // Check if li elements exist
    if ($li_elements->length === 0) {
        return $html; // Return the passed contents as is if <li> elements are not found
    }

    $result = '<ul>';

    // Iterate through li elements up to the specified count
    $i = 0;
    foreach ($li_elements as $element) {
        if ($i >= $count) {
            break;
        }
        $result .= '<li>' . $element->nodeValue . '</li>'; // Only add text content
        $i++;
    }

    $result .= '</ul>';
    return $result;
}


/**
 * Displays star ratings based on the given rating score, with full, partial, and empty stars.
 *
 * @param float $rating_score The rating score to be displayed as stars.
 * @return void This function echoes HTML <img> elements for displaying the stars.
 */
function quickaffilink_display_rating_stars_amazon($rating_score) {
    // Check if $rating_score is null or empty
    if (empty($rating_score)) {
        return; // Exit the function without outputting anything
    }

    $max_stars = 5; // Maximum number of stars
    $full_stars = floor($rating_score); // Number of full stars
    $partial_star = $rating_score - $full_stars; // Fractional part for partial star

    // Get the base URL of the plugin
    $base_url = plugins_url('/', plugin_dir_path(__FILE__));

    $full_star_url = $base_url . 'assets/images/rating-star-full.png';
    $partial_star_url = $base_url . 'assets/images/rating-star-partial.png';
    $empty_star_url = $base_url . 'assets/images/rating-star-empty.png';

    // Output full stars
    for ($i = 0; $i < $full_stars; $i++) {
        echo '<img src="' . esc_url($full_star_url) . '" alt="Full Star">';
    }

    // Output partial star if applicable
    if ($partial_star > 0) {
        echo '<img src="' . esc_url($partial_star_url) . '" alt="Partial Star">';
    }

    // Output empty stars for remaining space
    $empty_stars = $max_stars - $full_stars - ($partial_star > 0 ? 1 : 0);
    for ($i = 0; $i < $empty_stars; $i++) {
        echo '<img src="' . esc_url($empty_star_url) . '" alt="Empty Star">';
    }
}

/**
 * Extracts product information words from a given URL and returns a specified number of random words.
 *
 * @param string $url The URL from which product information words will be extracted.
 * @param int $numWords The number of product information words to return. Default is 3.
 * @return array An array containing product information words extracted from the URL.
 */
function quickaffilink_extractProductInfoFromUrl($url, $numWords = 3) {
    // Initialize an empty array to store product information words
    $productInfoWords = [];

    // Remove query parameters and fragments from the URL
    $url = preg_replace('/\?.*|#.*/', '', $url);

    // Split the URL by '/'
    $urlParts = explode('/', $url);

    // Find the index of the first non-empty segment
    $firstSegmentIndex = 0;
    foreach ($urlParts as $index => $part) {
        if (!empty($part)) {
            $firstSegmentIndex = $index;
            break;
        }
    }

    // Check if the URL contains a product identifier (e.g., "/dp/")
    if (in_array('dp', $urlParts)) {
        // Get the index of the "dp" segment
        $dpIndex = array_search('dp', $urlParts);
        // Extract the product name segment from the URL
        $productPath = $urlParts[$dpIndex + 1];
    } elseif (!empty($urlParts[$firstSegmentIndex + 1])) {
        // If no "dp" segment, use the segment right after the first non-empty segment
        $productPath = $urlParts[$firstSegmentIndex + 1];
    } else {
        // If no relevant segment found, return empty array
        return [];
    }

    // Split the product name segment into words
    $words = preg_split('/[-_\/]/', $productPath);

    // Filter out common words and add the rest to the product information words array
    foreach ($words as $word) {
        // Remove non-alphanumeric characters and convert to lowercase
        $word = preg_replace('/[^a-zA-Z0-9]/', '', $word);
        $word = strtolower($word);
        // Exclude short words and common words
        if (strlen($word) > 2 && !in_array($word, array('the', 'and', 'for', 'with', 'ref', 'sr', 'dp', 'th', 'va'))) {
            $productInfoWords[] = $word;
        }
    }

    // Remove duplicate words
    $productInfoWords = array_unique($productInfoWords);

    // Randomize the words' sequence
    shuffle($productInfoWords);

    // Return the specified number of words
    return array_slice($productInfoWords, 0, $numWords);
}


function quickaffilink_get_domain($url) {
	$parsed = wp_parse_url($url);
	$host = $parsed['host'];
    preg_match('/(?:https?:\/\/)?(?:www\.)?([^\/]+)/i', $host, $matches);
    return $matches[1];
}

/**
 * Formats the currency display based on the provided currency code or symbol.
 *
 * This function takes a currency code or symbol and a price as input and returns
 * a formatted string representing the price with the corresponding currency symbol.
 * If the provided currency code or symbol does not match any predefined symbol, it
 * returns the price followed by the currency code itself.
 * 
 * Currency-specific formatting:
 * - For AED (United Arab Emirates Dirham), the symbol comes after the price.
 * - For INR (Indian Rupee), the symbol is the Indian Rupee sign.
 * - For countries using $ as their currency symbol, the corresponding currency code (e.g., USD, AUD) is appended after the price.
 * 
 * Supported currencies and their symbols:
 * - USD: United States Dollar ($)
 * - CAD: Canadian Dollar (C$)
 * - MXN: Mexican Peso (MXN$)
 * - BRL: Brazilian Real (R$)
 * - GBP: British Pound Sterling (£)
 * - EUR: Euro (€)
 * - AUD: Australian Dollar (A$)
 * - INR: Indian Rupee (₹)
 * - SGD: Singapore Dollar (SG$)
 * - AED: United Arab Emirates Dirham (AED)
 * 
 * @param string $currency The currency code or symbol.
 * @param float $price The price to be formatted.
 * @return string The formatted price with the corresponding currency symbol or code.
 */
function quickaffilink_format_currency_display($currency, $price) {
    $formatted_price = '';
	
	$price = (float)$price;

    // Define currency symbols or codes for the provided countries
    $currency_symbols = array(
        'USD' => '$',   // USA
        'CAD' => 'C$',  // Canada
        'MXN' => 'MXN$', // Mexico
        'BRL' => 'R$',  // Brazil
        'GBP' => '£',   // United Kingdom
        'EUR' => '€',   // Germany, France, Italy, Spain, Netherlands
        'AUD' => 'A$',  // Australia
        'INR' => '₹',   // India
        'SGD' => 'SG$', // Singapore
        'AED' => 'AED', // United Arab Emirates
        // Add more currencies as needed
    );

    // Check if the provided input is a currency code or symbol
    if (array_key_exists($currency, $currency_symbols)) {
        $currency_symbol = $currency_symbols[$currency];
    } else {
        // If no matching symbol found, return the currency code
        $currency_symbol = $currency;
    }

    // Special handling for AED (United Arab Emirates Dirham)
    if ($currency === 'AED') {
        // Append the symbol after the price
        $formatted_price = number_format($price, 2) . ' ' . $currency_symbol;
    } elseif ($currency === 'INR') {
        // For INR (Indian Rupee), the symbol is the Indian Rupee sign
        $formatted_price = '₹' . number_format($price, 2);
    } elseif (strpos($currency_symbol, '$') !== false) {
        // For countries using $ as their currency symbol, append the currency code after the price
        $currency_code = array_search($currency_symbol, $currency_symbols); // Get the corresponding currency code
        $formatted_price = $currency_symbol . number_format($price, 2) . ' ' . $currency_code;
    } else {
        // Format the price with the currency symbol
        $formatted_price = $currency_symbol . number_format($price, 2);
    }

    return $formatted_price;
}

/**
 * Records the usage of a specific shortcode in the database.
 *
 * @param string $shortcode_name The name of the shortcode.
 * @return int|false The ID of the inserted or updated record, or false on failure.
 */
function quickaffilink_record_shortcode_usage($shortcode_name) {
    global $wpdb;
    $table_name_5 = $wpdb->prefix . 'quickaffilink_sc_stats';

    // Get the URL of the current page
	$url = isset($_SERVER['REQUEST_URI']) ? sanitize_text_field(wp_unslash($_SERVER['REQUEST_URI'])) : '';
    // Check if a record already exists for the shortcode and URL combination
    $existing_record = $wpdb->get_row(
        $wpdb->prepare(
            "SELECT * FROM $table_name_5 WHERE shortcode_name = %s AND url = %s",
            $shortcode_name,
            $url
        )
    );

    // If record already exists, update it
    if ($existing_record) {
        $updated = $wpdb->update(
            $table_name_5,
            array(
                'displayed_times' => $existing_record->displayed_times + 1,
                'last_seen' => current_time('mysql') // Update the last_seen timestamp
            ),
            array(
                'id' => $existing_record->id
            ),
            array(
                '%d', // 'displayed_times' is an integer
                '%s'  // 'last_seen' is a timestamp
            ),
            array(
                '%d' // 'id' is an integer
            )
        );

        if ($updated !== false) {
            return $existing_record->id; // Return the ID of the updated record
        } else {
            return false; // Return false on failure
        }
    } else {
        // Insert a new record into the shortcode usage table
        $inserted = $wpdb->insert(
            $table_name_5,
            array(
                'shortcode_name' => $shortcode_name,
                'url' => $url,
                'displayed_times' => 1,
                'first_detected' => current_time('mysql'), // Set the first_detected timestamp
                'last_seen' => current_time('mysql') // Set the last_seen timestamp
            ),
            array(
                '%s', // 'shortcode_name' is a string
                '%s', // 'url' is a string
                '%d', // 'displayed_times' is an integer
                '%s', // 'first_detected' is a timestamp
                '%s'  // 'last_seen' is a timestamp
            )
        );

        if ($inserted !== false) {
            return $wpdb->insert_id; // Return the ID of the inserted record
        } else {
            return false; // Return false on failure
        }
    }
}
/**/

/**
 * Returns the full name of an advertisement size variation based on its short name.
 *
 * @param string $short_name The short name representing the advertisement size variation.
 * @return string The full name of the advertisement size variation.
 */
function quickaffilink_fullShortcodeName($short_name){
    $full_name = "";

    // Determine the full name based on the short name
    switch ($short_name) {
        case 'sml_var1':
            $full_name = "Small (300x250)";
            break;
        case 'med_var1':
            $full_name = "Medium Var 1(480x320)";
            break;
        case 'med_var2':
            $full_name = "Medium Var 2(480x320)";
            break;
        case 'lar_var1':
            $full_name = "Large Var 1(1200x628)";
            break;
        case 'grp_var1':
            $full_name = "Multiple Products - Group";
            break;
        default:
            $full_name =$short_name;
    }

    return $full_name;
}


/**
 * Function to generate a time-based secure token.
 *
 * This function generates a time-based token that changes every 30 minutes.
 * The token includes an expiration time, allowing verification within 30 minutes.
 *
 * @return string Returns the generated time-based secure token.
 */
function quickaffilink_generate_time_based_token() {
    // Generate a random 8-byte token using cryptographic secure random bytes
    $randomBytes = random_bytes(8);

    // Get the current time and calculate the expiration time (30 minutes from now)
    $currentTime = time();
    $expirationTime = $currentTime + (30 * 60); // 30 minutes in seconds

    // Encode the token parts with their lengths
    $encodedToken = pack('NN', strlen($randomBytes), strlen($expirationTime)) . $randomBytes . $expirationTime;

    return bin2hex($encodedToken);
}

/**
 * Function to verify a time-based secure token.
 *
 * This function verifies if the provided token is valid and has not expired.
 * Tokens are valid within 30 minutes from the time they were generated.
 *
 * @param string $token The token to verify.
 *
 * @return bool Returns true if the token is valid and has not expired, false otherwise.
 */
function quickaffilink_verify_time_based_token($token) {
    // Get the current time
    $currentTime = time();

    // Decode the token from hexadecimal representation
    $encodedToken = hex2bin($token);

    // Extract the lengths of token parts
    list($randomBytesLength, $expirationTimeLength) = array_values(unpack('N*', substr($encodedToken, 0, 8)));

    // Extract the token parts
    $randomBytes = substr($encodedToken, 8, $randomBytesLength);
    $expirationTime = substr($encodedToken, 8 + $randomBytesLength, $expirationTimeLength);

    // Verify if the token is not expired (within 30 minutes)
    if ($currentTime <= $expirationTime) {
        return true;
    }

    return false;
}

/**
 * Function to verify if the request originated from the same domain.
 *
 * @return bool Returns true if the request originated from the same domain, false otherwise.
 */
function quickaffilink_verify_referral() {
    // Sanitize early: Clean and sanitize the referrer URL and server name
    $referrer_url = isset($_SERVER['HTTP_REFERER']) ? sanitize_text_field(wp_unslash($_SERVER['HTTP_REFERER'])) : '';
    $server_name = isset($_SERVER['SERVER_NAME']) ? sanitize_text_field(wp_unslash($_SERVER['SERVER_NAME'])) : '';

    // Validate: Check if the referrer URL is a valid URL
    if (!empty($referrer_url) && !filter_var($referrer_url, FILTER_VALIDATE_URL)) {
        return false; // Invalid referrer URL
    }

    // Parse the host from the sanitized referrer URL
    $referrer_host = wp_parse_url($referrer_url, PHP_URL_HOST);

    // Escape Late: Escape the server name before comparing to prevent XSS
    $server_host = esc_url($server_name);

    // Check if the sanitized referrer domain matches the escaped server domain
    if ($referrer_host === $server_host) {
        return true;
    }

    return false;
}
